# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright (C) 2014 Xavier Bruhiere


from celery import Celery
import os

from intuition.core.engine import Simulation
from intuition.utils.logger import get_nestedlog
import intuition.data.utils as datautils
import intuition.utils.dates as datesutils
import intuition.core.configuration as setup


broker_url = 'redis://{}:{}/0'.format(
    os.environ.get('BROKER_HOST', 'localhost'),
    os.environ.get('BROKER_PORT', 6379))

app = Celery('trades', backend=broker_url, broker=broker_url)

app.conf.update(
    CELERY_TASK_SERIALIZER='json',
    CELERY_ACCEPT_CONTENT=['json'],  # Ignore other content
    CELERY_RESULT_SERIALIZER='json',
    CELERY_TIMEZONE='Europe/Paris',
    CELERY_ENABLE_UTC=True,
)


@app.task
def trade(session, ctx, bot=False):
    log_setup = get_nestedlog(level='debug', show_log=False)
    with log_setup.applicationbound():

        configuration, strategy = setup.context(ctx)

        engine = Simulation()

        engine.configure_environment(
            configuration['index'][-1], configuration['exchange'])

        engine.build(session, configuration['modules'], strategy)

        data = {'universe': configuration['universe'],
                'index': configuration['index']}
        data.update(strategy['data'])
        is_live = datesutils.is_live(data['index'][-1])
        analyzes = engine.run(data, bot)

        # Get daily, cumulative and not, returns of portfolio and benchmark
        # NOTE Temporary fix before intuition would be able to get benchmark
        # data on live trading
        if not is_live:
            bm_sym = datautils.Exchanges[configuration['exchange']]['symbol']
            returns_df = analyzes.get_returns(benchmark=bm_sym)

        orders = 0
        for order in analyzes.results.orders:
            orders += len(order)

        final_value = analyzes.results.portfolio_value[-1]
        report = {
            'portfolio': final_value,
            'gain': final_value - strategy['manager']['cash'],
            'orders': orders,
            'pnl_mean': analyzes.results.pnl.mean(),
            'pnl_deviation': analyzes.results.pnl.std(),
        }
        if not is_live:
            report['portfolio_perfs'] = returns_df['algo_c_return'][-1] * 100.0
            report['benchmark_perfs'] = \
                returns_df['benchmark_c_return'][-1] * 100.0

        perfs = analyzes.overall_metrics('one_month')
        for k, v in perfs.iteritems():
            report[k] = v

        return report
